@@ -11,27 +11,56 @@ import logging
 from queue import Queue, Empty
 from http.client import HTTPException
 from .p3port import unicode
+from threading import Thread, Lock
+import time
 import typing
 import re
 import base64
 import simplenote
-from simplenote import Simplenote
 from . import events
+from . import utils
+from .debug import wrap_buggy_function
 
 # API key provided for nvPY.
 # Please do not use for other software!
 simplenote.simplenote.API_KEY = bytes(reversed(base64.b64decode('OTg0OTI4ZTg4YjY0NzMyOTZjYzQzY2IwMDI1OWFkMzg=')))
 
-from threading import Thread, Lock
-import time
-from . import utils
+
+# workaround for https://github.com/cpbotha/nvpy/issues/191
+class Simplenote(simplenote.Simplenote):
+    def get_token(self):
+        if self.token is None:
+            self.token = self.authenticate(self.username, self.password)
+            if self.token is None:
+                raise HTTPException('failed to connect to the server')
+        try:
+            return str(self.token, 'utf-8')
+        except TypeError:
+            return self.token
+
+    def get_note(self, *args, **kwargs):
+        try:
+            return super().get_note(*args, **kwargs)
+        except HTTPException as e:
+            return e, -1
+
+    def update_note(self, *args, **kwargs):
+        try:
+            return super().update_note(*args, **kwargs)
+        except HTTPException as e:
+            return e, -1
+
+    def get_note_list(self, *args, **kwargs):
+        try:
+            return super().get_note_list(*args, **kwargs)
+        except HTTPException as e:
+            return e, -1
+
 
 ACTION_SAVE = 0
 ACTION_SYNC_PARTIAL_TO_SERVER = 1
 ACTION_SYNC_PARTIAL_FROM_SERVER = 2  # UNUSED.
 
-from .debug import wrap_buggy_function
-
 
 class SyncError(RuntimeError):
     pass
@@ -82,7 +111,6 @@ class _BackgroundTaskReslt(typing.NamedTuple):
 class NotesDB(utils.SubjectMixin):
     """NotesDB will take care of the local notes database and syncing with SN.
     """
-
     def __init__(self, config):
         utils.SubjectMixin.__init__(self)
 
@@ -398,7 +426,8 @@ class NotesDB(utils.SubjectMixin):
                     # we have to store our local key also
                     filtered_notes.append(NoteInfo(key=k, note=n, tagfound=tagfound))
 
-        return filtered_notes, '|'.join(tms_pats[1] + tms_pats[2]), active_notes
+        match_regexp = '|'.join(re.escape(p) for p in tms_pats[1] + tms_pats[2])
+        return filtered_notes, match_regexp, active_notes
 
     def filter_notes_regexp(self, search_string=None):
         """Return list of notes filtered with search_string,
@@ -1044,84 +1073,81 @@ class NotesDB(utils.SubjectMixin):
         :return: UpdateResult object
         """
 
-        try:
+        self.waiting_for_simplenote = True
+        # WORKAROUND: simplenote <=v2.1.2 modifies the note passed by argument. To prevent on-memory database
+        #             corruption, Copy the note object before it is passed to simplenote library.
+        # https://github.com/cpbotha/nvpy/issues/181#issuecomment-489543782
+        o, err = self.simplenote.update_note(note.copy())
+        self.waiting_for_simplenote = False
+
+        if err == 0:
+            # success!
+
+            # Keeps the internal fields of nvpy.
+            new_note = dict(note)
+            new_note.update(o)
+
+            logging.debug('Server replies with updated note ' + new_note['key'])
+            return UpdateResult(
+                note=new_note,
+                is_updated=True,
+                error_object=None,
+            )
+
+        update_error = o
+
+        if 'key' in note:
+            # Note has already been saved on the simplenote server.
+            # Try to recover the update error.
             self.waiting_for_simplenote = True
-            # WORKAROUND: simplenote <=v2.1.2 modifies the note passed by argument. To prevent on-memory database
-            #             corruption, Copy the note object before it is passed to simplenote library.
-            # https://github.com/cpbotha/nvpy/issues/181#issuecomment-489543782
-            o, err = self.simplenote.update_note(note.copy())
+            o, err = self.simplenote.get_note(note['key'])
             self.waiting_for_simplenote = False
 
             if err == 0:
-                # success!
+                local_note = note
+                remote_note = o
 
-                # Keeps the internal fields of nvpy.
-                new_note = dict(note)
-                new_note.update(o)
-
-                logging.debug('Server replies with updated note ' + new_note['key'])
-                return UpdateResult(
-                    note=new_note,
-                    is_updated=True,
-                    error_object=None,
-                )
-
-            elif 'key' in note:
-                update_error = o
-
-                # note has already been saved on the simplenote server.
-                self.waiting_for_simplenote = True
-                o, err = self.simplenote.get_note(note['key'])
-                self.waiting_for_simplenote = False
-
-                if err == 0:
-                    local_note = note
-                    remote_note = o
-
-                    if not self.is_different_note(local_note, remote_note):
-                        # got an error response when updating the note.
-                        # however, the remote note has been updated.
-                        # this phenomenon is rarely occurs.
-                        # if it occurs, housekeeper's is going to repeatedly update this note.
-                        # regard updating error as success for prevent this problem.
-                        logging.info('Regard updating error (local key %s, error object %s) as success.' %
-                                     (o.key, repr(update_error)))
-                        return UpdateResult(
-                            note=local_note,
-                            is_updated=False,
-                            error_object=None,
-                        )
-
-                    else:
-                        # Local note and remote note are different.  But failed to update.
-                        logging.error('Could not update note %s to server: %s, local=%s, remote=%s' %
-                                      (note['key'], update_error, local_note, remote_note))
-                        return UpdateResult(
-                            note=None,
-                            is_updated=False,
-                            error_object=update_error,
-                        )
+                if not self.is_different_note(local_note, remote_note):
+                    # got an error response when updating the note.
+                    # however, the remote note has been updated.
+                    # this phenomenon is rarely occurs.
+                    # if it occurs, housekeeper's is going to repeatedly update this note.
+                    # regard updating error as success for prevent this problem.
+                    logging.info('Regard updating error (local key %s, error object %s) as success.' %
+                                 (o.key, repr(update_error)))
+                    return UpdateResult(
+                        note=local_note,
+                        is_updated=False,
+                        error_object=None,
+                    )
 
                 else:
-                    get_error = o
-                    logging.error('Could not get/update note %s: update_error=%s, get_error=%s' %
-                                  (note['key'], update_error, get_error))
+                    # Local note and remote note are different.  But failed to update.
+                    logging.error('Could not update note %s to server: %s, local=%s, remote=%s' %
+                                  (note['key'], update_error, local_note, remote_note))
                     return UpdateResult(
                         note=None,
                         is_updated=False,
-                        error_object={
-                            'update_error': update_error,
-                            'get_error': get_error
-                        },
+                        error_object=update_error,
                     )
 
-        except HTTPException as e:
-            # workaround for https://github.com/mrtazz/simplenote.py/issues/24
-            return UpdateResult(
-                note=None,
-                is_updated=False,
-                error_object=e,
-            )
+            else:
+                get_error = o
+                logging.error('Could not get/update note %s: update_error=%s, get_error=%s' %
+                              (note['key'], update_error, get_error))
+                return UpdateResult(
+                    note=None,
+                    is_updated=False,
+                    error_object={
+                        'update_error': update_error,
+                        'get_error': get_error
+                    },
+                )
+
+        # Failed to create new note.
+        assert err
+        assert 'key' not in note
+        return UpdateResult(note=None, is_updated=False, error_object=update_error)
 
 
 class Note(dict):
