@@ -48,6 +48,7 @@ from . import view
 import webbrowser
 from .version import VERSION
 from . import events
+from http.client import HTTPException
 
 try:
     import markdown
@@ -85,6 +86,8 @@ class ColorConfig(typing.NamedTuple):
     highlight_note_info: str
     # Text color for URL.
     url: str
+    # Background color for URL selection.
+    url_selection_background: str
     # Background color.
     background: str
     # Background color of highlighted area.
@@ -96,7 +99,6 @@ class Config:
     @ivar files_read: list of config files that were parsed.
     @ivar ok: True if config files had a default section, False otherwise.
     """
-
     def __init__(self, app_dir):
         """
         @param app_dir: the directory containing nvpy.py
@@ -126,6 +128,8 @@ class Config:
             'list_font_family': 'Helvetica',  # sans on all platforms
             'list_font_family_fixed': 'Courier',  # monospace on all platforms
             'list_font_size': '10',
+            'list_hide_time': '0',
+            'list_hide_tags': '0',
             'layout': 'horizontal',
             'print_columns': '0',
             'text_color': 'black',
@@ -133,6 +137,7 @@ class Config:
             'note_info_color': 'dark gray',
             'highlight_note_info_color': 'lightyellow',
             'url_color': '#03f',
+            'url_selection_background_color': 'yellow',
             'background_color': 'white',
             'highlight_background_color': 'yellow',
             'sn_username': '',
@@ -207,6 +212,9 @@ class Config:
         self.list_font_family_fixed = cp.get(cfg_sec, 'list_font_family_fixed')
         self.list_font_size = cp.getint(cfg_sec, 'list_font_size')
 
+        self.list_hide_time = cp.getint(cfg_sec, 'list_hide_time')
+        self.list_hide_tags = cp.getint(cfg_sec, 'list_hide_tags')
+
         self.layout = cp.get(cfg_sec, 'layout')
         self.print_columns = cp.getint(cfg_sec, 'print_columns')
 
@@ -216,6 +224,7 @@ class Config:
             note_info=cp.get(cfg_sec, 'note_info_color'),
             highlight_note_info=cp.get(cfg_sec, 'highlight_note_info_color'),
             url=cp.get(cfg_sec, 'url_color'),
+            url_selection_background=cp.get(cfg_sec, 'url_selection_background_color'),
             background=cp.get(cfg_sec, 'background_color'),
             highlight_background=cp.get(cfg_sec, 'highlight_background_color'),
         )
@@ -252,7 +261,6 @@ class NotesListModel(SubjectMixin):
     """
     @ivar list: List of (str key, dict note) objects.
     """
-
     def __init__(self):
         # call mixin ctor
         SubjectMixin.__init__(self)
@@ -285,7 +293,6 @@ class NotesListModel(SubjectMixin):
 class Controller:
     """Main application class.
     """
-
     def __init__(self, config):
         SubjectMixin.MAIN_THREAD = threading.current_thread()
 
@@ -450,7 +457,7 @@ class Controller:
     def observer_notes_db_error_sync_full(self, notes_db, evt_type, evt: events.SyncFailedEvent):
         try:
             raise evt.error
-        except SyncError as e:
+        except (SyncError, HTTPException) as e:
             self.view.show_error('Sync error', e)
         except WriteError as e:
             emsg = "Please check nvpy.log.\n" + str(e)
@@ -535,7 +542,7 @@ class Controller:
             logging.debug("Trying to convert %s to html." % (key, ))
             if HAVE_MARKDOWN:
                 logging.debug("Convert note %s to html." % (key, ))
-                exts = re.split("\\s", self.config.md_extensions.strip()) if self.config.md_extensions else []
+                exts = re.split('\s+', self.config.md_extensions.strip()) if self.config.md_extensions else []
                 exts += list(DEFAULT_MARKDOWN_EXTS)
                 # remove duplicate items on exts.
                 exts = list(set(exts))
@@ -668,7 +675,7 @@ class Controller:
                 self.view.update_selected_note_data(self.notes_db.notes[key])
                 self.view.set_status_text('Synced updated note from server.')
 
-            elif ret[1] == False:
+            elif ret and ret[1] == False:
                 self.view.set_status_text('Server had nothing newer for this note.')
 
             elif ret is None:
