@@ -10,15 +10,15 @@ from . import tk
 from tkinter import messagebox as tkMessageBox
 import tkinter.font as tkFont
 from . import utils
-import webbrowser
 import threading
 from . import events
 import typing
+import subprocess
+import platform
 
 
 class WidgetRedirector:
     """Support for redirecting arbitrary widget subcommands."""
-
     def __init__(self, widget):
         self.dict = {}
         self.widget = widget
@@ -101,7 +101,6 @@ class RedirectedText(tk.Text):
     This magic is encapsulated in the idlelib.WidgetRedirector class which
     we use here.
     """
-
     def __init__(self, master=None, cnf=None, **kw):
         tk.Text.__init__(self, master, cnf or {}, **kw)
 
@@ -125,7 +124,7 @@ class HelpBindings(tk.Toplevel):
         tk.Toplevel.__init__(self, parent)
         self.title("Help | Bindings")
 
-        import bindings
+        from . import bindings
 
         msg = tk.Text(self, width=80, wrap=tk.NONE)
         msg.insert(tk.END, bindings.description)
@@ -149,7 +148,6 @@ class SuggestionEntry(tk.Entry):
     args:
         completion_func(searchWord) - returns a list of matching tags.
     """
-
     def __init__(self, completion_func, *args, **kwargs):
         if "textvariable" in kwargs:
             self.var = kwargs["textvariable"]
@@ -312,20 +310,34 @@ class SuggestionEntry(tk.Entry):
 
 
 class TagList(tk.Toplevel):
-    def __init__(self, parent, taglist):
+    # search tag callback
+    def tagsel(self, box, view):
+        sel = box.curselection()
+        if len(sel) != 1: return
+        for idx in sel:
+            item = box.get(idx)
+            view.set_search_entry_text("t:%s" % item)
+        self.destroy()
+
+    def __init__(self, parent, taglist, view):
         tk.Toplevel.__init__(self, parent)
         self.title("List all tags")
+        self.bind("<Escape>", lambda a: self.destroy())
+
+        box = tk.Listbox(self, width=30, selectmode=tk.BROWSE)
         if taglist:
             alltags = list(set(taglist))
             alltags.sort(key=lambda x: x.upper())
-            tagtxt = '\n'.join(alltags)
+            for item in alltags:
+                box.insert(tk.END, item)
+            # bind double click and Enter (on box)
+            box.bind("<Double-Button-1>", lambda a: self.tagsel(box, view))
+            box.bind("<Return>", lambda a: self.tagsel(box, view))
+            box.focus()
         else:
-            tagtxt = "No tags defined"
+            box.insert(tk.END, "No tags defined")
 
-        msg = tk.Text(self, width=30, wrap=tk.NONE)
-        msg.insert(tk.END, tagtxt)
-        msg.config(state=tk.DISABLED)
-        msg.pack()
+        box.pack()
 
         button = tk.Button(self, text="Dismiss", command=self.destroy)
         button.pack()
@@ -401,6 +413,8 @@ class NotesListConfig(typing.NamedTuple):
     colors: str
     layout: str
     print_columns: int
+    hide_time: int
+    hide_tags: int
 
 
 class NoteConfig(typing.NamedTuple):
@@ -477,6 +491,8 @@ class NotesList(tk.Frame):
         self.note_headers = []
 
         self.layout = config.layout
+        self.hide_time = config.hide_time
+        self.hide_tags = config.hide_tags
         self.print_columns = config.print_columns
         if bold_font.measure(' ') > f.measure(' '):
             self.cwidth = bold_font.measure(' ')
@@ -499,15 +515,23 @@ class NotesList(tk.Frame):
         self.enable_text()
 
         if self.layout == "vertical" and self.print_columns == 1:
-            nrchars, rem = divmod((self.text.winfo_width()), self.cwidth)
-            cellwidth = (int(nrchars) - 8) / 2
+            nrchars, rem = divmod(self.text.winfo_width(), self.cwidth)
+            if self.hide_time:
+                nrchars = int(nrchars)
+            else:
+                nrchars = int(nrchars) - 8
+
+            if self.hide_tags:
+                cellwidth = int(nrchars)
+            else:
+                cellwidth = int(nrchars) / 2
 
             if pinned:
                 title += ' *'
 
             self.text.insert(tk.END, u'{0:<{w}}'.format(title[:cellwidth - 1], w=cellwidth), ("title,"))
 
-            if tags > 0:
+            if tags and not self.hide_tags:
                 if config.tagfound:
                     self.text.insert(tk.END, u'{0:<{w}}'.format(','.join(tags)[:cellwidth - 1], w=cellwidth),
                                      ("found", ))
@@ -515,7 +539,8 @@ class NotesList(tk.Frame):
                     self.text.insert(tk.END, u'{0:<{w}}'.format(','.join(tags)[:cellwidth - 1], w=cellwidth),
                                      ("tags", ))
 
-            self.text.insert(tk.END, ' ' + utils.human_date(createdate), ("createdate", ))
+            if not self.hide_time:
+                self.text.insert(tk.END, ' ' + utils.human_date(createdate), ("createdate", ))
 
             # tags can be None (newly created note) or [] or ['tag1', 'tag2']
         else:
@@ -527,10 +552,11 @@ class NotesList(tk.Frame):
             # latest modified first is the default mode
             # we could consider showing createddate here IF the sort mode
             # is configured to be latest created first
-            self.text.insert(tk.END, ' ' + utils.human_date(modifydate), ("modifydate", ))
+            if not self.hide_time:
+                self.text.insert(tk.END, ' ' + utils.human_date(modifydate), ("modifydate", ))
 
             # tags can be None (newly created note) or [] or ['tag1', 'tag2']
-            if tags:
+            if tags and not self.hide_tags:
                 if config.tagfound:
                     self.text.insert(tk.END, ' ' + ','.join(tags), ("found", ))
                 else:
@@ -544,6 +570,7 @@ class NotesList(tk.Frame):
         # Text widget events ##########################################
 
         self.text.bind("<Button 1>", self.cmd_text_button1)
+        self.text.bind("<Control-c>", self.cmd_text_copy)
 
         # same deal as for pageup
         # we have to stop the text widget class event handler from firing
@@ -582,6 +609,12 @@ class NotesList(tk.Frame):
         idx = int(text_index.split('.')[0]) - 1
         self.select(idx, silent=False)
 
+    def cmd_text_copy(self, event):
+        if self.selected_idx >= 0:
+            self.text.clipboard_clear()
+            self.text.clipboard_append(self.get_title(self.selected_idx))
+        return "break"
+
     def clear(self):
         """
 
@@ -729,7 +762,6 @@ class TriggeredcompleteEntry(tk.Entry):
 
     @ivar cycle: if 1, then we're cycling through alternative completions.
     """
-
     def __init__(self, master, case_sensitive, **kw):
         tk.Entry.__init__(self, master, **kw)
         self.case_sensitive = case_sensitive
@@ -815,7 +847,6 @@ class TriggeredcompleteText(RedirectedText):
     TriggeredcompleteText completes the note title when I press "[[".
     This class behaves like the TriggeredcompleteEntry.
     """
-
     def __init__(self, master, case_sensitive, **kwargs):
         RedirectedText.__init__(self, master, **kwargs)
         self.case_sensitive = case_sensitive
@@ -953,7 +984,6 @@ class TriggeredcompleteText(RedirectedText):
 class View(utils.SubjectMixin):
     """Main user interface class.
     """
-
     def __init__(self, config, notes_list_model):
         utils.SubjectMixin.__init__(self)
 
@@ -1181,6 +1211,8 @@ class View(utils.SubjectMixin):
         self.text_note.bind("<Control-bracketleft>", lambda e: self.notes_list.text.focus())
         # <Key>
 
+        self.text_note.bind("<Control-BackSpace>", self.handler_control_backspace)
+        self.text_note.bind("<Control-Delete>", self.handler_control_delete)
         self.text_note.bind("<Control-a>", self.cmd_select_all)
 
         self.tags_entry.bind("<Return>", self.handler_add_tags_to_selected_note)
@@ -1374,6 +1406,8 @@ class View(utils.SubjectMixin):
 
         notes_list_config = NotesListConfig(colors=self.config.colors,
                                             layout=self.config.layout,
+                                            hide_time=self.config.list_hide_time,
+                                            hide_tags=self.config.list_hide_tags,
                                             print_columns=self.config.print_columns)
 
         if self.config.layout == "horizontal":
@@ -1559,15 +1593,15 @@ class View(utils.SubjectMixin):
         self.text_note.event_generate('<<Paste>>')
 
     def cmd_help_about(self):
-
         tkMessageBox.showinfo('Help | About',
-                              'nvPY %s is copyright 2012-2016 by Charl P. Botha '
-                              '<http://charlbotha.com/>\n\n'
-                              'A rather ugly but cross-platform simplenote client.' % (self.config.app_version, ),
+                              'nvPY %s - A rather ugly but cross-platform simplenote client.\n\n'
+                              'Copyright 2017-2020 yuuki0xff <https://yuuki0xff.jp/>\n'
+                              'Copyright 2012-2016 Charl P. Botha <http://charlbotha.com/>\n' %
+                              (self.config.app_version, ),
                               parent=self.root)
 
     def cmd_list_tags(self):
-        l = TagList(self.root, self.taglist)
+        l = TagList(self.root, self.taglist, self)
         self.root.wait_window(l)
 
     def cmd_help_bindings(self):
@@ -1734,7 +1768,12 @@ class View(utils.SubjectMixin):
             self.notify_observers('click:notelink', link)
 
         else:
-            webbrowser.open(link)
+            if platform.system().lower() == 'windows':
+                os.startfile(link)
+            elif platform.system().lower() == 'darwin':
+                subprocess.call(('open', link))
+            else:
+                subprocess.call(('xdg-open', link))
 
     def activate_search_string_highlights(self):
         # no note selected, so no highlights.
@@ -1783,40 +1822,55 @@ class View(utils.SubjectMixin):
         """
 
         t = self.text_note
-        # the last group matches [[bla bla]] inter-note links
-        pat = \
-        r"\b((https?|ftp|file)://[-A-Za-z0-9+&@#/%?=~_|!:,.;\(\)]*[A-Za-z0-9+&@#/%=~_|])|(\[\[[^][]*\]\])"
 
-        # remove all existing tags
+        # Remove all existing tags
         for tag in self.text_tags_links:
             t.tag_remove(tag, '1.0', 'end')
 
         del self.text_tags_links[:]
 
+        # List of Regex patterns to match various link types to be activated
+        re_list = [
+            r"(\[\[[^][]*\]\])",  # Inter-note Links
+            r"((?:https?|ftp|file)://[-A-Za-z0-9+&@#/%?=~_|!:,.;\(\)]*[A-Za-z0-9+&@#/%=~_|])",  # Http(s) / FTP / File Links
+            r"(mailto:[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+)",  # Mailto Links
+            r"((?:tel|mid):[^\s]+)",  # Tel / Mid Links
+            r"((?:thunderlink|irc|ircs|irc6)://[^\s]+)",  # Thunderbird and IRC Links
+            r"(message:(?://)?(?:%3c|<).*(?:%3e|>))"  # Leopard Mail Message Links
+        ]
+
+        # Compile Regex into single pattern
+        pat = re.compile('|'.join(re_list))
+
         for mo in re.finditer(pat, t.get('1.0', 'end')):
-            # extract the link from the match object
-            if mo.groups()[2] is not None:
-                link = mo.groups()[2]
+            # Any Match in Group 0 is a Note Link
+            if mo.groups()[0] is not None:
+                link = mo.groups()[0]
                 ul = 0
             else:
-                link = mo.groups()[0]
+                link = list(filter(None, mo.groups()))[0]
                 ul = 1
 
-            # start creating a new tkinter text tag
+            # Create a new Tkinter Tag
             tag = 'web-%d' % (len(self.text_tags_links), )
-            t.tag_config(tag, foreground=self.config.colors.url, underline=ul)
-            # hovering should give us the finger (cursor) hehe
+            t.tag_config(tag,
+                         foreground=self.config.colors.url,
+                         selectbackground=self.config.colors.url_selection_background,
+                         underline=ul)
+
+            # Hovering over link changes cursor to hand
             t.tag_bind(tag, '<Enter>', lambda e: t.config(cursor="hand2"))
             t.tag_bind(tag, '<Leave>', lambda e: t.config(cursor=""))
-            # and clicking on it should do something sensible
-            t.tag_bind(tag, '<Button-1>', lambda e, link=link: self.handler_click_link(link))
 
-            # mo.start(), mo.end() or mo.span() in one go
-            t.tag_add(tag, '1.0+%dc' % (mo.start(), ), '1.0+%dc' % (mo.end(), ))
+            # Clicking link calls link handler method
+            t.tag_bind(tag, '<Button-1>', lambda e, link=link: self.handler_click_link(link))
 
             # record the tag name so we can delete it later
             self.text_tags_links.append(tag)
 
+            # mo.start(), mo.end() or mo.span() in one go
+            t.tag_add(tag, '1.0+%dc' % (mo.start(), ), '1.0+%dc' % (mo.end(), ))
+
     def activate_markdown_highlighting(self):
         t = self.text_note
         content = t.get('1.0', 'end')
@@ -1838,6 +1892,58 @@ class View(utils.SubjectMixin):
             # mo.start(), mo.end() or mo.span() in one go
             t.tag_add('md-bold', '1.0+{0}c'.format(mo.start()), '1.0+{0}c'.format(mo.end()))
 
+    def handler_control_backspace(self, evt):
+        if self.text_note.index("insert-1c") != "1.0" and self.text_note.index(tk.INSERT) != "1.0":
+            if self.text_note.index("insert wordstart-1c") == "1.0":
+                i = 0
+                while self.text_note.index(tk.INSERT) != "1.0" and i < 25:
+                    self.text_note.delete("insert-1c")
+                    i += 1
+            else:
+                insertm1c = self.text_note.get("insert-1c")
+                while insertm1c in "  " and insertm1c != "":
+                    self.text_note.delete("insert-1c")
+                    insertm1c = self.text_note.get("insert-1c")
+                if insertm1c in "\n\t.?!,@#…¿/\\\"'—–" and insertm1c != "":
+                    i = 0
+                    while (insertm1c in "\n\t.?!,@#…¿/\\\"'—–" and insertm1c != "") and i < 25:
+                        self.text_note.delete("insert-1c")
+                        insertm1c = self.text_note.get("insert-1c")
+                        i += 1
+                else:
+                    while insertm1c not in "\n\t  .?!,@#…¿/\\\"'—–" and insertm1c != "":
+                        self.text_note.delete("insert-1c")
+                        insertm1c = self.text_note.get("insert-1c")
+        return "break"
+
+    def handler_control_delete(self, evt):
+        insert = self.text_note.get(tk.INSERT)
+        if insert in "  " and insert != "":
+            while insert in "  " and insert != "":
+                self.text_note.delete(tk.INSERT)
+                insert = self.text_note.get(tk.INSERT)
+        elif insert not in "\n\t  .?!,@#…¿/\\\"'—–":
+            while insert not in "\n\t  .?!,@#…¿/\\\"'—–":
+                self.text_note.delete(tk.INSERT)
+                insert = self.text_note.get(tk.INSERT)
+            if insert in "  ":
+                while insert in "  " and insert != "":
+                    self.text_note.delete(tk.INSERT)
+                    insert = self.text_note.get(tk.INSERT)
+        else:
+            if insert in "\n\t" and insert != "":
+                if self.text_note.get(
+                        tk.INSERT,
+                        tk.END).strip() == "" and self.text_note.index(tk.INSERT) != self.text_note.index(tk.END):
+                    self.text_note.delete(tk.INSERT, tk.END)
+                else:
+                    while insert in "\n\t  " and insert != "":
+                        self.text_note.delete(tk.INSERT)
+                        insert = self.text_note.get(tk.INSERT)
+            else:
+                self.text_note.delete(tk.INSERT)
+        return "break"
+
     def handler_text_change(self, evt):
         self.notify_observers('change:text', None)
         # FIXME: consider having this called from the housekeeping
